// This example code explains how to use the structures 
// and all the functions of the tool library

// Let's include the tool library
#include "tools.h"

int main()
{
 // Initialize seed for random numbers (using cpu clock)
 Randomize(); 

 /////////////////////////////////////////////////////////
 // TEST FOR MATRICES
 
 // Declare a matrix A
 Matrix A;
 
 // Initialize A with 5 rows and 3 columns
 initMatrix(&A,5,3);
 
 // Fill A with random entries ranging in (-10,5)
 fillMatrix(&A,-10,5);
 
 // Write A on the file A.dat 
 writeMatrix(&A,"A.dat");
 
 // Print A on video
 printMatrix(&A);

 // Set A=0 (all entries are zero)
 resetMatrix(&A);
 
 // Print A on video again
 printMatrix(&A);
 
 // Erase A from memory (A now needs to be re-initialized!)
 freeMatrix(&A);
 
 // Initialize A with 6 rows and 6 columns
 initMatrix(&A,6,6);
 
 // Fill A with random entries ranging in (2,2) ... all entries are 2!
 fillMatrix(&A,2,2);

 // Print A on video again
 printMatrix(&A);
 
 // Erase A from memory again (A now needs to be re-initialized!)
 freeMatrix(&A);
 
 // Load A from file A.dat (A is automatically re-initialized!)
 readMatrix(&A,"A.dat");
 
 // Print A on video again
 printMatrix(&A);

 /////////////////////////////////////////////////////////
 // TEST FOR VECTORS
 
 // Declare a vector v
 Vector v;

 // Initialize v with 5 elements
 initVector(&v,5);
 
 // Fill v with random elements ranging in (-10,10)
 fillVector(&v,-10,10);
 
 // Write v on the file v.dat 
 writeVector(&v,"v.dat");
 
 // print v on video 
 printVector(&v);
 
 // Set v=0 (all elements are zero)
 resetVector(&v);
 
 // print v on video again
 printVector(&v);

 // Erase v from memory (v now needs to be re-initialized!)
 freeVector(&v);
 
 // Initialize v with 10 elements
 initVector(&v,10);
 
 // Fill v with random elements ranging in (3,3) ... all entries are 3!
 fillVector(&v,3,3);
 
 // print v on video again
 printVector(&v);
 
 // Erase v from memory again (v now needs to be re-initialized!)
 freeVector(&v);
 
 // Load v from file v.dat (v is automatically re-initialized!)
 readVector(&v,"v.dat");
 
 // Print v on video again
 printVector(&v);

 /////////////////////////////////////////////////////////
 // Write gnuplot data for drawing 
 // the graph of the function f(x)=sin(2pi x)exp(-x)
 // using vectors
 
 // Declare vectors for x coordinates and corresponding values f(x)
 Vector x, Fx;

 // Define the number of nodes for the discretization
 int i, nodes=101;

 // Define the domain [xmin,xmax] for f and the space step dx
 double xmin=0, xmax=2, dx=(xmax-xmin)/(nodes-1);
 
 // Initialize x and Fx with nodes elements
 initVector(&x,nodes);
 initVector(&Fx,nodes);
 
 // Fill x and Fx with the corresponding values
 for(i=0;i<nodes;i++)
 {
   x.data[i]=xmin+i*dx;
  Fx.data[i]=sin(2*M_PI*x.data[i])*exp(-x.data[i]);
 }
 
 // Write data on the file Fx.gnp
 gnuplotVector(&x,&Fx,"Fx.gnp"); 
 // To draw data open gnuplot and type "load Fx.gnp"
 
 /////////////////////////////////////////////////////////
 // Write gnuplot data for drawing 
 // the graph of the function f(x,y)=sin(2pi sqrt(x^2+y^2) )exp(- sqrt(x^2+y^2) )
 // using vectors and matrices
 
 // Declare vectors for x and y coordinates
 Vector X, Y;
 
 // Declare a matrix for the corresponding values f(x,y)
 Matrix FXY;
 
 // Define the number of nodes for the discretization in x and y
 int I, J, nodesX=51, nodesY=51;

 // Define the domain [Xmin,Xmax]x[Ymin,Ymax] for f and the space steps dX and dY
 double Xmin=-2, Xmax=2, dX=(Xmax-Xmin)/(nodesX-1);
 double Ymin=-2, Ymax=2, dY=(Ymax-Ymin)/(nodesY-1);

 // Define an auxiliary variable for storing sqrt(x^2+y^2)
 double R;
 
 // Initialize X, Y and FXY respectively with 
 // nodesX, nodesY and nodesX*nodesY elements
 initVector(&X,nodesX);
 initVector(&Y,nodesY);
 
 // Note that Y is used for rows and X for columns!!!
 initMatrix(&FXY,nodesY,nodesX);
 
 // Fill X, Y and FXY with the corresponding values
 for(I=0;I<nodesY;I++)
 {
  Y.data[I]=Ymin+I*dY;
  
  for(J=0;J<nodesX;J++)
  {
   X.data[J]=Xmin+J*dX;
   
   R=sqrt(X.data[J]*X.data[J]+Y.data[I]*Y.data[I]);
   
   FXY.row[I][J]=sin(2*M_PI*R)*exp(-R);
  }
 }

 // Write data on the file Fx.gnp
 gnuplotMatrix(&X,&Y,&FXY,"FXY.gnp");
 // To draw data open gnuplot and type "load FXY.gnp"
 
 /////////////////////////////////////////////////////////
 
 return 0; 
}




