#include "tools.h"

/////////////////////////////////////////////////////////

void Randomize(){srand(time(0));}

/////////////////////////////////////////////////////////

double Random(double rmin, double rmax)
{
 return rmin+((double)rand()/RAND_MAX)*(rmax-rmin);
}

/////////////////////////////////////////////////////////

void initVector(Vector *v, int n)
{
 v->size=n;
 v->data=MALLOC(double,n);
}

/////////////////////////////////////////////////////////

void freeVector(Vector *v)
{
 v->size=0;
 free(v->data);
}

/////////////////////////////////////////////////////////

void fillVector(Vector *v, double rmin, double rmax)
{
 int i;
 for(i=0; i<v->size; i++)
  v->data[i]=Random(rmin,rmax);
}

/////////////////////////////////////////////////////////

double norm2(Vector *v)
{
 double out=0;  
 int i;
 for(i=0; i<v->size; i++)
  out += v->data[i]*v->data[i];
 out=sqrt(out);
 return out;
}

/////////////////////////////////////////////////////////

void printVector(Vector *v)
{
 int i;
 printf("Vector: size = %d\n",v->size); 
 printf("( "); 
 for(i=0;i<v->size;i++)
  printf("%lf ",v->data[i]);
 printf(")\n"); 
 printf("Norm 2 = %lf\n",norm2(v));
 printf("\n");
}

/////////////////////////////////////////////////////////

void resetVector(Vector *v)
{
 int i;
 for(i=0;i<v->size;i++)
  v->data[i]=0;  
}

/////////////////////////////////////////////////////////

void writeVector(Vector *v, const char* filename)
{
 FILE *pf;
 int i;

 pf=fopen(filename,"w");

 fprintf(pf,"#VECTOR\n");

 fprintf(pf,"%d\n",v->size);
 
 for(i=0;i<v->size;i++)
  fprintf(pf,"%lf\n",v->data[i]);

 fclose(pf);
}

/////////////////////////////////////////////////////////

void gnuplotVector(Vector *x, Vector *v,const char *filename)
{
 FILE *pf;
 int i;
 
 pf=fopen(filename,"w");
 
 fprintf(pf,"plot \"-\" w l lw 2 lc rgb '#0' title \"%s\"\n",filename);
 
 if(x && x->size == v->size)
 {
  for(i=0;i<v->size;i++) fprintf(pf,"%lf %lf\n",x->data[i],v->data[i]);
 }
 else
 {
  for(i=0;i<v->size;i++) fprintf(pf,"%d %lf\n",i,v->data[i]);
 }
 
 fprintf(pf,"e\n");
 fclose(pf);
 printf("File %s written. Plot with \"load %s\" in gnuplot!\n",filename,filename);
 printf("\n");
}

/////////////////////////////////////////////////////////

void readVector(Vector *v, const char* filename)
{
 FILE *pf;
 int i;
 char header[10];
 
 pf=fopen(filename,"r");

 if(!pf)
 {
  printf("Cannot open %s: file not found\n",filename); exit(0);  
 }
 
 fscanf(pf,"%s",&header);
 
 if(!strcmp(header,"#VECTOR"))
 {
  fscanf(pf,"%d",&i);
  initVector(v,i);
  for(i=0;i<v->size;i++)
   fscanf(pf,"%lf",v->data+i);
 }
 else
 {
  printf("Error in %s: not a Vector file\n",filename); exit(0);  
 }
 fclose(pf);
}

/////////////////////////////////////////////////////////

void initMatrix(Matrix *M, int n, int m)
{
 M->rows=n;
 M->cols=m; 
 M->data=MALLOC(double,n*m); 
 M->row=MALLOC(double *,n);
 int i;
 for(i=0; i<n; i++) M->row[i]=M->data+i*m;
}

/////////////////////////////////////////////////////////

void freeMatrix(Matrix *M)
{
 int i;
 free(M->row); 
 free(M->data);
 M->rows=0;
 M->cols=0;
}

/////////////////////////////////////////////////////////

void printMatrix(Matrix *M)
{
 int i,j;
 printf("Matrix: size = %d x %d\n",M->rows,M->cols); 
 for(i=0;i<M->rows;i++)
 {
  printf("| ");
  for(j=0;j<M->cols;j++)
   printf("%s%5.5lf |",((M->row[i][j]>0) ? " " : ""),M->row[i][j]);  
  printf("\n");
 }
 printf("\n");
}

/////////////////////////////////////////////////////////

void fillMatrix(Matrix *M, double rmin, double rmax)
{
 int i,j;
 for(i=0; i<M->rows; i++)
   for(j=0; j<M->cols; j++)
    M->row[i][j]=Random(rmin,rmax);
}

/////////////////////////////////////////////////////////

void resetMatrix(Matrix *M)
{
 int i,j;
 for(i=0; i<M->rows; i++)
   for(j=0; j<M->cols; j++)
    M->row[i][j]=0;
}

/////////////////////////////////////////////////////////

void writeMatrix(Matrix *M, const char* filename)
{
 FILE *pf;
 int i,j;

 pf=fopen(filename,"w");

 fprintf(pf,"#MATRIX\n");

 fprintf(pf,"%d %d\n",M->rows,M->cols);
 
 for(i=0;i<M->rows;i++)
 {
  for(j=0;j<M->cols;j++)
   fprintf(pf,"%lf ",M->row[i][j]);
  fprintf(pf,"\n");
 }
 fclose(pf);
}

/////////////////////////////////////////////////////////

void gnuplotMatrix(Vector *x, Vector *y, Matrix *M, const char *filename)
{
 FILE *pf;
 int i,j;
 
 pf=fopen(filename,"w");
 
 fprintf(pf,"set hidden3d; splot \"-\" w l lc rgb '#0' title \"%s\"\n",filename);
 
 if(x && y && x->size == M->cols && y->size == M->rows)
 {
  for(i=0;i<M->rows;i++)
  {
   for(j=0;j<M->cols;j++)
    fprintf(pf,"%lf %lf %lf\n",x->data[j],y->data[i],M->row[i][j]);
   fprintf(pf,"\n");
  }
 }
 else
 {
  for(i=0;i<M->rows;i++)
  {
   for(j=0;j<M->cols;j++)
    fprintf(pf,"%d %d %lf\n",j,i,M->row[i][j]);
   fprintf(pf,"\n");
  }
 }
 
 fprintf(pf,"e\n");
 fclose(pf);
 printf("File %s written. Plot with \"load %s\" in gnuplot!\n",filename,filename);
 printf("\n");
}

/////////////////////////////////////////////////////////

void readMatrix(Matrix *M, const char* filename)
{
 FILE *pf=0;
 int i,j;
 char header[10];
 
 pf=fopen(filename,"r");

 if(!pf)
 {
  printf("Cannot open %s: file not found\n",filename); exit(0);  
 }
 
 fscanf(pf,"%s\n",&header);
 
 if(!strcmp(header,"#MATRIX"))
 {
  fscanf(pf,"%d %d",&i,&j);
  initMatrix(M,i,j);
  for(i=0;i<M->rows;i++)
   for(j=0;j<M->cols;j++)
   fscanf(pf,"%lf",&M->row[i][j]);
 }
 else
 {
  printf("Error in %s: not a Matrix file\n",filename); exit(0);  
 }
 fclose(pf);
}

/////////////////////////////////////////////////////////
